#include <sys/types.h>

#include "../include/os.h"
#ifdef __MSW__
# include <windows.h>
#endif
#include <GL/gl.h>
#include <GL/glu.h>

#include "gw.h"
#include "stategl.h"
#include "obj.h"
#include "sar.h"

#include "sardrawdefs.h"


extern void SARVertexBoxNS(double x_len, double y_len, double z_len);
extern void SARVertexBoxBaseNS(double x_len, double y_len, double z_len);


void SARDrawHumanIterate(
        gw_display_struct *display, sar_scene_struct *scene,
        double height, double mass,
        sar_obj_flags_t flags,          /* Human flags. */
        sar_color_struct *palette,      /* Human colors. */
        int water_ripple_tex_num,
        sar_grad_anim_t anim_pos
);
void SARDrawHuman(
        gw_display_struct *display, sar_scene_struct *scene,
        sar_object_struct *obj_ptr,
        sar_object_human_struct *obj_human_ptr
);


/*
 *	Called by SARDrawHuman() to draw one human.
 *
 *	All inputs assumed valid.
 */
void SARDrawHumanIterate(
	gw_display_struct *display, sar_scene_struct *scene,
	double height, double mass,
	sar_obj_flags_t flags,		/* Human flags. */
	sar_color_struct *palette,	/* Human colors. */
	int water_ripple_tex_num,
	sar_grad_anim_t anim_pos
)
{
	sar_color_struct *c;
	GLenum shade_model_mode;

	double anim_coeff = (double)anim_pos /
	    (double)((sar_grad_anim_t)-1);

	double torso_angle;
	double left_bisep_angle, right_bisep_angle;
	double left_shoulder_angle, right_shoulder_angle;
	double left_trisep_angle, right_trisep_angle;
	double left_hip_to_thigh_angle, right_hip_to_thigh_angle;
	double left_knee_to_calv_angle, right_knee_to_calv_angle;

	double thigh_len = 0.3;
	double calv_len = 0.4;
	double torso_len = 0.8;
	double bisep_len = 0.3;
	double trisep_len = 0.4;
	double base_to_torso = 0.0;	/* Ground to bottom of torso
					 * (calculated later).
					 */
	double streatcher_height = 0.8;	/* Ground to bed of streatcher. */

#define SET_COLOR	\
{ \
 glColor4f( \
  (GLfloat)c->r, \
  (GLfloat)c->g, \
  (GLfloat)c->b, \
  (GLfloat)c->a \
 ); \
}

	/* No texture. */
	V3DTextureSelect(NULL);

        /* Set shade model dependant of global options. */
        shade_model_mode = display->state_gl.shade_model_mode;
        StateGLShadeModel(&display->state_gl, option.gl_shade_model);


	/* Calculate angles of body parts. Note that all zero radians
	 * indicates human is standing upright with hands to side.
	 */
	if(flags & (SAR_HUMAN_FLAG_RUN_AWAY | SAR_HUMAN_FLAG_RUN_TOWARDS))
	{
            if(anim_coeff > 0.5)
                anim_coeff = (1.0 - anim_coeff) / 0.5;
            else
                anim_coeff = anim_coeff / 0.5;

            torso_angle = 0.1 * PI;
            left_shoulder_angle = 0.0 * PI;
            right_shoulder_angle = 0.0 * PI;
	    if(flags & SAR_HUMAN_FLAG_PUSHING)
	    {
		left_bisep_angle = (1.7 * PI);
		right_bisep_angle = (1.7 * PI);
	    }
	    else
	    {
		left_bisep_angle = (1.7 + (0.35 * (1.0 - anim_coeff))) * PI;
		right_bisep_angle = (1.7 + (0.35 * anim_coeff)) * PI;
	    }
            left_trisep_angle = 1.8 * PI;
            right_trisep_angle = 1.8 * PI;
            left_hip_to_thigh_angle = (0.2 - (0.5 * anim_coeff)) * PI;
            right_hip_to_thigh_angle = (0.2 - (0.5 * (1.0 - anim_coeff))) * PI;
            left_knee_to_calv_angle = 0.2 * PI;
            right_knee_to_calv_angle = 0.2 * PI;

            base_to_torso = thigh_len + calv_len;
	}
	else if(flags & SAR_HUMAN_FLAG_LYING)
	{
            torso_angle = 1.5 * PI;
	    left_shoulder_angle = 0.0 * PI;
	    right_shoulder_angle = 0.0 * PI;
            left_bisep_angle = 0.0 * PI;
	    right_bisep_angle = 0.0 * PI;
            left_trisep_angle = 0.0 * PI;
            right_trisep_angle = 0.0 * PI;
            left_hip_to_thigh_angle = 0.0 * PI;
            right_hip_to_thigh_angle = 0.0 * PI;
            left_knee_to_calv_angle = 0.0 * PI;
            right_knee_to_calv_angle = 0.0 * PI;

            base_to_torso = thigh_len + calv_len;

	    if(flags & SAR_HUMAN_FLAG_ON_STREATCHER)
	    {
		/* Some translating will be done later. */
	    }
	}
        else if(flags & SAR_HUMAN_FLAG_SIT)
        {
	    /* Sitting base at tush. */
            torso_angle = 0.0 * PI;
            left_shoulder_angle = 0.0 * PI;
            right_shoulder_angle = 0.0 * PI;
            left_bisep_angle = 0.0 * PI;
            right_bisep_angle = 0.0 * PI;
            left_trisep_angle = 1.75 * PI;
            right_trisep_angle = 1.75 * PI;
            left_hip_to_thigh_angle = 1.5 * PI;
            right_hip_to_thigh_angle = 1.5 * PI;
            left_knee_to_calv_angle = 0.5 * PI;
            right_knee_to_calv_angle = 0.5 * PI;

            base_to_torso = 0;
        }
        else if(flags & SAR_HUMAN_FLAG_SIT_DOWN)
        {
            torso_angle = 0.0 * PI;
            left_shoulder_angle = 0.0 * PI;
            right_shoulder_angle = 0.0 * PI;
            left_bisep_angle = 0.0 * PI;
            right_bisep_angle = 0.0 * PI;
            left_trisep_angle = 1.75 * PI;
            right_trisep_angle = 1.75 * PI;
            left_hip_to_thigh_angle = 1.5 * PI;
            right_hip_to_thigh_angle = 1.5 * PI;
            left_knee_to_calv_angle = 0.0 * PI;
            right_knee_to_calv_angle = 0.0 * PI;

            base_to_torso = 0;
	}
        else if(flags & SAR_HUMAN_FLAG_SIT_UP)
        {
	    /* Sitting base at feet. */
            torso_angle = 0.0 * PI;
            left_shoulder_angle = 0.0 * PI;
            right_shoulder_angle = 0.0 * PI;
            left_bisep_angle = 0.0 * PI;
            right_bisep_angle = 0.0 * PI;
            left_trisep_angle = 1.75 * PI;
            right_trisep_angle = 1.75 * PI;
            left_hip_to_thigh_angle = 1.5 * PI;
            right_hip_to_thigh_angle = 1.5 * PI; 
            left_knee_to_calv_angle = 0.5 * PI;
            right_knee_to_calv_angle = 0.5 * PI;
        
            base_to_torso = calv_len;
	}
	else if(flags & SAR_HUMAN_FLAG_DIVER_CATCHER)
	{
	    /* Diver catching victim, used for hoist rescue end drawing. */
	    torso_angle = 0.0 * PI;
	    left_shoulder_angle = 0.1 * PI;
	    right_shoulder_angle = 1.9 * PI;
	    left_bisep_angle = 1.8 * PI;
	    right_bisep_angle = 1.8 * PI;
	    left_trisep_angle = 1.8 * PI;   
	    right_trisep_angle = 1.8 * PI;
	    left_hip_to_thigh_angle = 1.9 * PI;
	    right_hip_to_thigh_angle = 1.9 * PI;
	    left_knee_to_calv_angle = 0.2 * PI; 
	    right_knee_to_calv_angle = 0.2 * PI;

	    /* If gripped it implies on end of rescue hoist rope. */
	    if(flags & SAR_HUMAN_FLAG_GRIPPED)
		base_to_torso = torso_len;
	    else if(flags & SAR_HUMAN_FLAG_IN_WATER)
		base_to_torso = torso_len * -0.92;
	    else
		base_to_torso = thigh_len + calv_len;
	}
	else
	{
	    /* Standing. */
	    if(flags & SAR_HUMAN_FLAG_IN_WATER)
	    {
		/* Standing in water, arms fore and moving. */
		if(anim_coeff > 0.5)
		    anim_coeff = (1.0 - anim_coeff) / 0.5;
		else
		    anim_coeff = anim_coeff / 0.5;

                torso_angle = 0.0 * PI;
                left_shoulder_angle = (0.4 * anim_coeff) * PI;
                right_shoulder_angle = (2 - (0.4 * anim_coeff)) * PI;
                left_bisep_angle = 1.5 * PI;
                right_bisep_angle = 1.5 * PI;
                left_trisep_angle = 0.0 * PI;
	        right_trisep_angle = 0.0 * PI;
                left_hip_to_thigh_angle = 0.0 * PI;
                right_hip_to_thigh_angle = 0.0 * PI;
                left_knee_to_calv_angle = 0.0 * PI;
                right_knee_to_calv_angle = 0.0 * PI;

		/* Since in water, move torso down. */
		base_to_torso = torso_len * -0.92;
	    }
	    else
	    {
		/* Standing on ground. */
		if((flags & SAR_HUMAN_FLAG_NEED_RESCUE) &&
		   (flags & SAR_HUMAN_FLAG_AWARE)
		)
		{
                 /* Need rescue and aware that rescuer is
		  * near by, move arms fore and wave.
		  */
                 if(anim_coeff > 0.5)
                    anim_coeff = (1.0 - anim_coeff) / 0.5;
                 else
                    anim_coeff = anim_coeff / 0.5;

                 torso_angle = 0.0 * PI;
                 left_shoulder_angle = (0.7 + (0.4 * anim_coeff)) * PI; 
                 right_shoulder_angle = (1.3 - (0.4 * anim_coeff)) * PI;
                 left_bisep_angle = 0.0 * PI; 
                 right_bisep_angle = 0.0 * PI; 
                 left_trisep_angle = 0.0 * PI; 
                 right_trisep_angle = 0.0 * PI;
                 left_hip_to_thigh_angle = 0.0 * PI; 
                 right_hip_to_thigh_angle = 0.0 * PI;  
                 left_knee_to_calv_angle = 0.0 * PI; 
                 right_knee_to_calv_angle = 0.0 * PI;

                 base_to_torso = thigh_len + calv_len;
		}
		else
		{
		 /* Just standing on ground. */
                 torso_angle = 0.0 * PI;
                 left_shoulder_angle = 0.0 * PI;
                 right_shoulder_angle = 0.0 * PI;
                 left_bisep_angle = 0.0 * PI;
                 right_bisep_angle = 0.0 * PI;
                 left_trisep_angle = 0.0 * PI;
                 right_trisep_angle = 0.0 * PI;
                 left_hip_to_thigh_angle = 0.0 * PI;
                 right_hip_to_thigh_angle = 0.0 * PI;
                 left_knee_to_calv_angle = 0.0 * PI;
                 right_knee_to_calv_angle = 0.0 * PI;

		 base_to_torso = thigh_len + calv_len;
		}
	    }
	}


	/* Begin drawing human. */

	/* Torso. */
        glPushMatrix();
        {
            /* Check if on streatcher, if so translate up. */
            if(flags & SAR_HUMAN_FLAG_ON_STREATCHER)  
            {
		/* Push on extra matrix and move up (matrix poped later). */
		glPushMatrix();
                glTranslatef(0, (GLfloat)streatcher_height, 0);
            }

	    /* Move to base of torso (the tush). */
            glRotatef((GLfloat)-SFMRadiansToDegrees(torso_angle), 1.0, 0.0, 0.0);
            glTranslatef(0, (GLfloat)base_to_torso, 0);

	    /* Move to center of torso and draw torso. */
            glPushMatrix();
            {
		/* Hips of torso. */
		c = &palette[SAR_HUMAN_COLOR_HIPS];
		SET_COLOR
                glBegin(GL_QUADS);
                {
	            SARVertexBoxBaseNS(
			0.45 * torso_len,
			0.25 * torso_len,
			0.10 * torso_len
		    );
                }
                glEnd();


		glTranslatef(0, (GLfloat)(0.10 * torso_len), 0);

                /* Upper torso. */
                c = &palette[SAR_HUMAN_COLOR_TORSO];
                SET_COLOR
                glBegin(GL_QUADS);
                {
                    SARVertexBoxBaseNS(
                        0.45 * torso_len,
                        0.25 * torso_len,
                        0.90 * torso_len
                    );
                }
                glEnd();
	    }
	    glPopMatrix();
	    /* Back to base of torso. */

	    /* Draw left leg. */
            glPushMatrix();
            {
		/* Thigh. */
		glRotatef((GLfloat)-SFMRadiansToDegrees(left_hip_to_thigh_angle), 1.0, 0.0, 0.0);
                glTranslatef((GLfloat)(-0.16 * torso_len), 0, 0);

                c = &palette[SAR_HUMAN_COLOR_LEGS];
                SET_COLOR

                glBegin(GL_QUADS);
                {
                    SARVertexBoxBaseNS(
                        (GLfloat)(0.5 * thigh_len),
                        (GLfloat)(0.5 * thigh_len),
                        (GLfloat)(-thigh_len)
                    );
                }
                glEnd();

		/* Calv. */
                glTranslatef(0, (GLfloat)-thigh_len, 0);
		glRotatef((GLfloat)-SFMRadiansToDegrees(left_knee_to_calv_angle), 1.0, 0.0, 0.0);

		/* Use same color as for thighs. */

                glBegin(GL_QUADS);
                {
                    SARVertexBoxBaseNS(
                        (GLfloat)(0.3 * calv_len),
                        (GLfloat)(0.3 * calv_len),
                        (GLfloat)-calv_len
                    );
                }
                glEnd();

		/* Feet. */
                glTranslatef(0, (GLfloat)-calv_len, (GLfloat)-0.05);

                c = &palette[SAR_HUMAN_COLOR_FEET];
                SET_COLOR

                glBegin(GL_QUADS);
                {
                    SARVertexBoxBaseNS(
                        0.1,
                        0.2,
                        -0.1
                    );
                }
                glEnd();
	    }
            glPopMatrix();
            /* Back to base of torso. */

            /* Draw right leg. */
            glPushMatrix();
            {
                /* Thigh. */
		glRotatef((GLfloat)-SFMRadiansToDegrees(right_hip_to_thigh_angle), 1.0, 0.0, 0.0);
                glTranslatef((GLfloat)(0.16 * torso_len), 0, 0);

                c = &palette[SAR_HUMAN_COLOR_LEGS];
                SET_COLOR

                glBegin(GL_QUADS);
                {
                    SARVertexBoxBaseNS(
                        (GLfloat)(0.5 * thigh_len),
                        (GLfloat)(0.5 * thigh_len),
                        (GLfloat)-thigh_len
                    );  
                }
                glEnd();   
                
                /* Calv. */
                glTranslatef(0, (GLfloat)-thigh_len, 0);
		glRotatef((GLfloat)-SFMRadiansToDegrees(right_knee_to_calv_angle), 1.0, 0.0, 0.0);

		/* Use same color as for thighs. */

                glBegin(GL_QUADS);
                {
                    SARVertexBoxBaseNS(
                        (GLfloat)(0.3 * calv_len),
                        (GLfloat)(0.3 * calv_len),
                        (GLfloat)-calv_len
                    );
                }
                glEnd();

                /* Feet. */
                glTranslatef(0, (GLfloat)-calv_len, (GLfloat)-0.05);

                c = &palette[SAR_HUMAN_COLOR_FEET];
                SET_COLOR

                glBegin(GL_QUADS);
                {
                    SARVertexBoxBaseNS(
                        0.1,
                        0.2,
                        -0.1
                    );
                }
                glEnd();
            }
            glPopMatrix();
            /* Back to base of torso. */

	    /* Move to shoulders. */
	    glTranslatef(0, (GLfloat)(0.95 * torso_len), 0);

            /* Draw left arm. */
            glPushMatrix();
            {
                /* Bisep and shoulder. */
		glTranslatef(
		    (GLfloat)((-0.225 * torso_len) + (-0.25 * bisep_len)),
		    0,
		    0
		);
                glRotatef((GLfloat)-SFMRadiansToDegrees(left_bisep_angle), 1.0, 0.0, 0.0);
		glRotatef((GLfloat)-SFMRadiansToDegrees(left_shoulder_angle), 0.0, 0.0, 1.0);

                c = &palette[SAR_HUMAN_COLOR_ARMS];
                SET_COLOR

                glBegin(GL_QUADS);
                {
                    SARVertexBoxBaseNS(
                        0.5 * bisep_len,
                        0.5 * bisep_len,
                        -bisep_len
                    );
                }
                glEnd();

                /* Trisep. */
                glTranslatef(0, (GLfloat)-bisep_len, 0);
		glRotatef((GLfloat)-SFMRadiansToDegrees(left_trisep_angle), 1.0, 0.0, 0.0);

                glBegin(GL_QUADS);
                {
                    SARVertexBoxBaseNS(
                        0.28 * trisep_len,
                        0.28 * trisep_len,
                        -trisep_len
                    );
                }
                glEnd();


                /* Hand. */
                glTranslatef(0, (GLfloat)-trisep_len, 0);

                c = &palette[SAR_HUMAN_COLOR_HANDS];
                SET_COLOR

                glBegin(GL_QUADS);
                {
                    SARVertexBoxBaseNS(0.2, 0.1, -0.2);
                }
                glEnd();
	    }
            glPopMatrix();
            /* Back to shoulders. */

	    /* Draw right arm. */
            glPushMatrix();
            {
                /* Bisep. */
                glTranslatef(
		    (GLfloat)((0.225 * torso_len) + (0.25 * bisep_len)),
		    0,
		    0
		);
		glRotatef((GLfloat)-SFMRadiansToDegrees(right_bisep_angle), 1.0, 0.0, 0.0);
		glRotatef((GLfloat)-SFMRadiansToDegrees(right_shoulder_angle), 0.0, 0.0, 1.0);

                c = &palette[SAR_HUMAN_COLOR_ARMS];
                SET_COLOR

                glBegin(GL_QUADS);
                {
                    SARVertexBoxBaseNS(
                        0.5 * bisep_len,
                        0.5 * bisep_len,
                        -bisep_len
                    );
                }
                glEnd();

                /* Trisep. */
                glTranslatef(0, (GLfloat)-bisep_len, 0);
		glRotatef((GLfloat)-SFMRadiansToDegrees(right_trisep_angle), 1.0, 0.0, 0.0);

                glBegin(GL_QUADS);
                {
                    SARVertexBoxBaseNS(
                        (GLfloat)(0.28 * trisep_len),
                        (GLfloat)(0.28 * trisep_len),
                        (GLfloat)-trisep_len
                    );
                }
                glEnd();


		/* Hand. */
                glTranslatef(0, (GLfloat)-trisep_len, 0);

                c = &palette[SAR_HUMAN_COLOR_HANDS];
                SET_COLOR

		glBegin(GL_QUADS);
                {
                    SARVertexBoxBaseNS(0.2, 0.1, -0.2);
                }
                glEnd();
            }
            glPopMatrix();
            /* Back to shoulders. */


            /* Head. */
            glTranslatef(0, (GLfloat)(0.05 * torso_len), 0);

            c = &palette[SAR_HUMAN_COLOR_FACE];
            SET_COLOR

            glBegin(GL_QUADS);
            {
                SARVertexBoxBaseNS(0.18, 0.18, 0.25);
            }
            glEnd();


	    /* Hair. */
	    glPushMatrix();
	    {
                glTranslatef(0, 0.25, 0);

                c = &palette[SAR_HUMAN_COLOR_HAIR];
                SET_COLOR

                glBegin(GL_QUADS);
                {
                    SARVertexBoxBaseNS(0.18, 0.18, 0.1);
                }
                glEnd();
	    }
	    glPopMatrix();
	    /* Back of head hair. */
            glPushMatrix();
            {
                glTranslatef((GLfloat)0.0, (GLfloat)0.05, (GLfloat)(-(-0.09 - 0.03)));
                glBegin(GL_QUADS);
                {
                    SARVertexBoxBaseNS(0.18, 0.06, 0.25);
                }
                glEnd();
            }
            glPopMatrix();


            /* Check if on streatcher, if so we need to pop one matrix
	     * level that was added for translation up on to streatcher.
	     */
            if(flags & SAR_HUMAN_FLAG_ON_STREATCHER)
            {
                glPopMatrix();
            }
        }
        glPopMatrix();

	/* Draw water ripples if in water. */
	if(flags & SAR_HUMAN_FLAG_IN_WATER)
	{
	    GLboolean depth_mask_flag;
	    int tex_num, frame_num;
	    v3d_texture_ref_struct *t;
	    double hr = 5.0;	/* Half ripple radius. */


	    tex_num = water_ripple_tex_num;
            if(SARIsTextureAllocated(scene, tex_num) &&
	       option.textured_objects
	    )
	    {
		t = scene->texture_ref[tex_num];

		/* Select frame by animation, re-update anim_coeff. */
		anim_coeff = (double)anim_pos /
                    (double)((sar_grad_anim_t)-1);

		frame_num = (int)(anim_coeff * t->total_frames);
		if(frame_num >= t->total_frames)
		    frame_num = t->total_frames - 1;
		if(frame_num < 0)
		    frame_num = 0;
		V3DTextureSelectFrame(t, frame_num);

		glColor4f(1.0, 1.0, 1.0, 0.5);

                StateGLEnable(&display->state_gl, GL_BLEND);
                StateGLDisable(&display->state_gl, GL_ALPHA_TEST);
                depth_mask_flag = display->state_gl.depth_mask_flag;
                StateGLDepthMask(&display->state_gl, GL_FALSE);
		StateGLEnable(&display->state_gl, GL_POLYGON_OFFSET_FILL);
		StateGLBlendFunc(
		    &display->state_gl, GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA
		);
		StateGLPolygonOffset(
		    &display->state_gl,
		    (GLfloat)option.gl_polygon_offset_factor, -1.0
		);

		glBegin(GL_QUADS);
		{
		    glNormal3f(0, 1, 0);
                    glTexCoord2f(0, 1 - 0);
                    glVertex3f((GLfloat)-hr, 0, (GLfloat)hr);
                    glTexCoord2f(1, 1 - 0);
                    glVertex3f((GLfloat)hr, 0, (GLfloat)hr);
                    glTexCoord2f(1, 1 - 1);
                    glVertex3f((GLfloat)hr, 0, (GLfloat)-hr);
                    glTexCoord2f(0, 1 - 1);
                    glVertex3f((GLfloat)-hr, 0, (GLfloat)-hr);
		}
		glEnd();

                StateGLDisable(&display->state_gl, GL_POLYGON_OFFSET_FILL);
                StateGLDisable(&display->state_gl, GL_BLEND);
		StateGLDepthMask(&display->state_gl, depth_mask_flag);
                StateGLEnable(&display->state_gl, GL_ALPHA_TEST);
	    }
	}

        /* Restore to previous shade model. */
        StateGLShadeModel(&display->state_gl, shade_model_mode);

#undef SET_COLOR

	return;
}


/*
 *      Draws a human specified by the object obj_ptr.
 *
 *      The human substructure obj_human_ptr should be the one on
 *      the object.
 *
 *      All inputs are assumed valid.
 */
void SARDrawHuman(
        gw_display_struct *display, sar_scene_struct *scene,
        sar_object_struct *obj_ptr,
        sar_object_human_struct *obj_human_ptr
)
{
	SARDrawHumanIterate(
	    display, scene,
	    obj_human_ptr->height, obj_human_ptr->mass,
	    obj_human_ptr->flags, obj_human_ptr->color,
	    obj_human_ptr->water_ripple_tex_num,
	    obj_human_ptr->anim_pos
	);

	return;
}
