#include <stdio.h>
#include <string.h>
#include <sys/types.h>

#include "../include/os.h"

#ifdef __MSW__
# include <windows.h>
#endif

#include <GL/gl.h>

#include "v3dtex.h"

#include "obj.h"
#include "smoke.h"


int SmokeCreatePosition(
	sar_scene_struct *scene,
	sar_object_struct ***ptr, int *total,
	sar_position_struct *pos,
	double radius,
	const char *tex_name,
	time_t life_span
);


/*
 *	Creates a smoke puff object, returning the new object's number
 *	or -1 on error.
 */
int SmokeCreatePosition(
	sar_scene_struct *scene,
	sar_object_struct ***ptr, int *total,
	sar_position_struct *pos,
	double radius,
	const char *tex_name,
	time_t life_span
)
{
	int obj_num;
	sar_object_struct *obj_ptr;
	sar_object_smoke_struct *obj_smoke_ptr;


	if((scene == NULL) ||
           (pos == NULL) ||
	   (tex_name == NULL)
	)
	    return(-1);

	/* Create smoke puff object. */
	obj_num = SARObjCreate(
	    scene, ptr, total,
	    SAR_OBJ_TYPE_SMOKE
	);
	if(obj_num < 0)
	    return(-1);

	obj_ptr = (*ptr)[obj_num];
	if(obj_ptr == NULL)
	    return(-1);

	/* Set new values. */
	memcpy(
	    &obj_ptr->pos,
	    pos,
	    sizeof(sar_position_struct)
	);
	obj_ptr->range = 7000;	/* Visable about 5 miles. */

	SARObjAddContactBoundsSpherical(
	    obj_ptr,
	    0, 0,	/* Crash flags and crash type. */
	    radius
	);

	obj_ptr->life_span = life_span;


        /* Get pointer to smoke puff sub structure. */
        obj_smoke_ptr = (sar_object_smoke_struct *)obj_ptr->data;
        if(obj_smoke_ptr == NULL)
            return(-1);

	obj_smoke_ptr->radius = radius;
	obj_smoke_ptr->tex_num = SARGetTextureRefNumberByName(
            scene, tex_name
        );


	return(obj_num);
}
